#!perl
# Based on boggle2.pl by Hubert Feyrer
#
# Copyright (c) 2019 Hubert Feyrer <hubert@feyrer.de>
# Alle Rechte vorbehalten.

$|++; # autoflush stdout - good for |tee 

$N=9;
#$DICTFILE="deutsch.txt";
$DICTFILE="GC80EC9_norros-9-der-wurfel.dict";

$maxlen = -1;

###########################################################################
sub rotN {
    my ( $word, $n )  = @_;
    my $res="";
    my $x;

    foreach $c ( split(//, uc($word))) {
       $x = ord($c) - ord('A');
       $x += $n;
       $x = $x % 26;
       $x += ord('A');
       $x=chr($x);
       #print "c=$c, x=$x\n";
       $res .= $x;
    }

    return $res;
}

###########################################################################
sub decode_chain {
    my ( $chain )  = @_;
    $word="";
    foreach $pair (split(/-/, $chain)) {
	($x,$y) = $pair =~ /\((\d+),(\d+)\)/;;
	$word .= $field[$x][$y];
	#print "decode_chain($chain): word=$word (x=$x,y=$y)\n";
    }
    return $word;
}

###########################################################################
sub chain {
    my ($chain, $x, $y, $depth) = @_;
    if ($chain =~ /\($x,$y\)/) {
        # char already used, skip
	#print "SKIP $x,$y - already in $chain\n";
	return;
    }
    
    $chain .= "-($x,$y)";
    if ($depth > 0) {
	# check intermediate word
	$chain =~ s,^-,,;
	my $word = decode_chain($chain);
	my $score = $dict{$word};
	if ($score >0) {
	    $found{$word} = $score;
	    $found_chain{$word} = $chain;
	    #print "found1: $word: $score - $chain\n";
	}

	# recursion
	chain("$chain", $x-1, $y-1, $depth-1) if $x>0 and $y>0;
	chain("$chain", $x,   $y-1, $depth-1) if $y>0;
	chain("$chain", $x+1, $y-1, $depth-1) if $x<$N-1 and $y>0;
	
	chain("$chain", $x-1, $y, $depth-1) if $x>0;
	chain("$chain", $x+1, $y, $depth-1) if $x<$N-1;
	
	chain("$chain", $x-1, $y+1, $depth-1) if $x>0 and $y<$N-1;
	chain("$chain", $x,   $y+1, $depth-1) if $y<$N-1;
	chain("$chain", $x+1, $y+1, $depth-1) if $x<$N-1 and $y<$N-1;
    } else {
	# end recursion
	$chain =~ s,^-,,;
	my $word = decode_chain($chain);
	my $score = $dict{$word};
	if ($score > 0) {
	    $found{$word} = $score;
	    $found_chain{$word} = $chain;
	    #print "found2: $word: $score - $chain\n";
	}
    }
}

###########################################################################
###########################################################################
### M A I N
###########################################################################
###########################################################################

# Init field
for($i=0; $i<$N; $i++) {
    $line=<DATA>;
    for($j=0; $j<$N; $j++) {
	($c, $rest) = $line =~ /^(.)(.*)$/;
	$field[$i][$j] = $c;
	$line=$rest;
    }
}

# print field
for($i=0; $i<$N; $i++) {
    for($j=0; $j<$N; $j++) {
	print $field[$i][$j]." ";
    }
    print "\n";
}
print "\n";

# read dict
open(DICT, $DICTFILE) or die "Cannot open dictfile $DICTFILE: $!\n";
while($w = <DICT>) {
    if ($w =~ /[\/+-]/) {
        #print "SKIP: $w\n";
	next;
    }
    
    chomp($w);
    my $l = length($w);
    my $score = -1;
    
    if (length($w) >= 3) { # min wordlength
	if ($l == 3) {      $score = 1;
	} elsif ($l == 4) { $score = 1;
	} elsif ($l == 5) { $score = 2;
	} elsif ($l == 6) { $score = 3;
	} elsif ($l == 7) { $score = 5;
	} else {	    $score = 11;
	}
	
	for($n=0; $n<26; $n++) {
		$rot=rotN(uc($w), $n);
		$dict{$rot} = $n;
		#print "DICT: $rot\n";
	}

	$maxlen = $l if $l > $maxlen;
    }
}
close(DICT);
$maxlen = ($maxlen>$N*$N)?$N*$N:$maxlen;
print "maxlen=$maxlen\n";

$maxlen = 8;  # hack

# walk field
$todo=$N*$N;
for(my $i=0; $i<$N; $i++) {
    for(my $j=0; $j<$N; $j++) {
	print "todo: $todo @ ".localtime()."\n";
	$todo--;
	
	#print "$i,$j: \n";
	chain("", $i,$j, $maxlen-1)."\n";
	#print "\n";
    }
    #print "\n";
}

# print findings
my $totakscore = 0;
foreach $key (sort keys %found) {
    $score = $found{$key};
    $chain = $found_chain{$key};
    print "$key: $score - $chain\n";
    $totalscore+=$score;
}
print "total score: $totalscore\n";

#<>; # hello windows


__DATA__
UHTVAMHIR
HUQNRSYZE
UNDHMROPA
ODHLOCHIN
BXUCGBFKP
XWCSJQHHO
KTQLEIPGG
OBNHOFFEO
TFMNORROS
